from os.path import join, dirname
from textx import metamodel_from_file
from textx.export import metamodel_export, model_export
from robonav.robot import Robot

def navigator(debug=False):

    this_folder = dirname(__file__)

    robot_mm = metamodel_from_file(join(this_folder, 'robot.tx'), debug=False)

    robot_model = robot_mm.model_from_file(join(this_folder, 'program.rbt'))

    robot = Robot(robot_model)
    robot.interpret()


# if __name__ == "__main__":
#     navigator()



class Robot(object):

    def __init__(self,model):
        self.model = model
        # Initial position is (0,0)
        self.x = 0
        self.y = 0

    def print_menu(self):
        print(f"Current location: {self.x},{self.y}")
        print("List of available directions:")
        for i,command in enumerate(self.model.commands):
            print(i+1,'-', command.direction)
        print('q - quit')
        print("Input the number correponding to the direction:")

    def get_inputs(self):
        steps=input()
        try:
            if steps == 'q':
                print('No movement')
                return
            else:
                return steps
        except Exception:
            print('Invalid input')
        

    def command(self,comm):
        if comm.direction=='initial':
            print('Give co-ordinates in x,y format')
            received_inputs = self.get_inputs()
            steps=received_inputs if received_inputs is not None else '0,0'
            self.x=int(steps.split(',')[0])
            self.y=int(steps.split(',')[1])
        else:
            print(f"How many steps you want to go {comm.direction}")
            received_inputs = self.get_inputs()
            steps=received_inputs if received_inputs is not None else '0'
            steps=int(steps)
            move = {
                    "up": (0, 1),
                    "down": (0, -1),
                    "left": (-1, 0),
                    "right": (1, 0)
                }[comm.direction]
            # Calculate new robot position
            self.x += steps * move[0]
            self.y += steps * move[1]       

    def __str__(self):
        return f"Robot position is {self.x}, {self.y}."

    def interpret(self):
        self.print_menu()
        while True:
            try:
                command=input()
                if command == 'q':
                    return
                elif int(command)<=len(self.model.commands):                    
                    command = int(command)
                    command = self.model.commands[command - 1] 
                    self.command(command)
                else:
                    print('Invalid input')              
                    
            except Exception:
                print('Something happened!')
            
            self.print_menu()       

            



from setuptools import setup, find_packages

VERSION = '0.0.3'
DESCRIPTION = 'A simple robot runner'
LONG_DESCRIPTION = 'A package that allows to control a robot using custom language.'

# Setting up
setup(
    name="robonav",
    version=VERSION,
    author="LiveByTheCode (Archer EarthX)",
    author_email="<archerearthx@gmail.com>",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=LONG_DESCRIPTION,
    packages=find_packages(),
    install_requires=['textx'],
    include_package_data=True,
    package_data={"": ["*.tx","*.rbt"]},
    entry_points={
          'console_scripts': [
              'robonav=robonav:navigator',
          ]
    },
    keywords=['python', 'textx', 'dsl', 'robot', 'state machine'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: End Users/Desktop",
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7'
)